import 'dart:async';
import 'dart:io';

import 'package:data_mock/database/avatar_model_dao.dart';
import 'package:data_mock/database/image_model_dao.dart';
import 'package:data_mock/mock_mixin.dart';
import 'package:data_mock/model/avatar_model.dart';
import 'package:data_mock/model/image_model.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:json2dart_safe/json2dart.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';

class MockDbManager extends BaseDbManager with MockMixin {
  @override
  FutureOr<void> onCreate(Database db, int version) async {
    await db.execute(AvatarModelDao.tableSql());
    await db.execute(ImageModelDao.tableSql());
  }

  @override
  Future<void> init() async {
    if (kDebugMode) {
      var databasesPath = await getDatabasesPath();
      String path = join(databasesPath, getDbName());
      File _dbFile = File(path);
      if (!_dbFile.existsSync()) {
        try {
          await Directory(dirname(path)).create(recursive: true);
        } catch (_) {}

        //AssetBundle bundle = AssetBundle();
        ByteData data = await rootBundle.load('packages/data_mock/assets/client_db.db');
        List<int> bytes = data.buffer.asUint8List(data.offsetInBytes, data.lengthInBytes);
        _dbFile.writeAsBytes(bytes, flush: true);
      }
    }
    super.init();
  }

  ImageModelDao? _imageModelDao;

  ImageModelDao get imageModelDao => _imageModelDao ??= ImageModelDao();

  AvatarModelDao? _avatarModelDao;

  AvatarModelDao get avatarModelDao => _avatarModelDao ??= AvatarModelDao();

  @override
  Future<String?> queryAvatar() async => (await avatarModelDao.random("avatars"))?.avatar;

  @override
  Future<List<String>?> queryAvatars(int count) async {
    List<AvatarModel>? images = await avatarModelDao.randoms(count, "avatars");
    return images?.map((e) => e.avatar ?? "").toList();
  }

  @override
  Future<String?> queryImage() async => (await imageModelDao.random("images"))?.image;

  @override
  Future<List<String>?> queryImages(int count) async {
    List<ImageModel>? images = await imageModelDao.randoms(count, "images");
    return images?.map((e) => e.image ?? "").toList();
  }
}
